@@ -20,27 +20,30 @@ from difflib import SequenceMatcher
 from pickle import dumps
 
 
-class TypeHandler(object):
-    """
-    Base class for type handlers.
+class TypeHandler():
+    """Base class for type handlers.
 
     Handlers provide diff, patch, generate_formatted_diff and iterate_diff
     methods for specific type.
 
     """
 
+    extension_id = None
     handled_type = None
 
     type_prefix = ''
     type_suffix = ''
 
     def diff(self, differ, a, b):
-        """
-        Return equality flag and diff for two objects.
+        """Calculate diff for two objects.
 
-        :param differ: nested_diff.Differ object.
-        :param a: First dict to diff.
-        :param b: Second dict to diff.
+        Args:
+            differ: nested_diff.Differ object.
+            a: First object to diff.
+            b: Second object to diff.
+
+        Returns:
+            Tuple: equality flag and nested diff.
 
         """
         diff = {}
@@ -59,12 +62,18 @@ class TypeHandler(object):
         return equal, diff
 
     def patch(self, patcher, target, diff):
-        """
-        Return patched object.
+        """Patch object.
 
-        :param patcher: nested_diff.Patcher object.
-        :param target: Object to patch.
-        :param diff: Nested diff.
+        Args:
+            patcher: nested_diff.Patcher object.
+            target: Object to patch.
+            diff: Nested diff.
+
+        Returns:
+            Patched object.
+
+        Raises:
+            ValueError: Inapropriate diff tag found.
 
         """
         try:
@@ -76,11 +85,14 @@ class TypeHandler(object):
         raise ValueError(diff)
 
     def iterate_diff(self, iterator, diff):
-        """
-        Iterate over nested diff.
+        """Iterate over nested diff.
 
-        :param patcher: nested_diff.Iterator object.
-        :param diff: Nested diff.
+        Args:
+            iterator: nested_diff.Iterator object.
+            diff: Nested diff.
+
+        Yields:
+            Tuple: diff, None, None
 
         """
         yield diff, None, None
@@ -100,12 +112,15 @@ class ScalarHandler(TypeHandler):
     """Base class for scalar handlers."""
 
     def diff(self, differ, a, b):
-        """
-        Return diff for two scalar objects.
+        """Calculate diff for for two scalar objects.
 
-        :param differ: nested_diff.Differ object.
-        :param a: First dict to diff.
-        :param b: Second dict to diff.
+        Args:
+            differ: nested_diff.Differ object.
+            a: First object to diff.
+            b: Second object to diff.
+
+        Returns:
+            Tuple: equality flag and nested diff.
 
         """
         diff = {}
@@ -151,18 +166,22 @@ class BytesHandler(ScalarHandler):
 class DictHandler(TypeHandler):
     """dict handler."""
 
+    extension_id = 0
     handled_type = dict
 
     type_prefix = '{'
     type_suffix = '}'
 
     def diff(self, differ, a, b):
-        """
-        Return equality flag and diff for two dicts.
+        """Calculate diff for two dict objects.
 
-        :param differ: nested_diff.Differ object.
-        :param a: First dict to diff.
-        :param b: Second dict to diff.
+        Args:
+            differ: nested_diff.Differ object.
+            a: First dict to diff.
+            b: Second dict to diff.
+
+        Returns:
+            Tuple: equality flag and nested diff.
 
         >>> from nested_diff import Differ
         >>>
@@ -213,12 +232,15 @@ class DictHandler(TypeHandler):
         return equal, diff
 
     def patch(self, patcher, target, diff):
-        """
-        Return patched dict.
+        """Patch dict object.
 
-        :param patcher: nested_diff.Patcher object.
-        :param target: dict to patch.
-        :param diff: nested diff.
+        Args:
+            patcher: nested_diff.Patcher object.
+            target: dict object to patch.
+            diff: Nested diff.
+
+        Returns:
+            Patched dict.
 
         """
         for key, subdiff in diff['D'].items():
@@ -232,11 +254,14 @@ class DictHandler(TypeHandler):
         return target
 
     def iterate_diff(self, iterator, diff):
-        """
-        Iterate over dict diff.
+        """Iterate over dict diff.
 
-        :param iterator: nested_diff.Iterator object.
-        :param diff: nested diff.
+        Args:
+            iterator: nested_diff.Iterator object.
+            diff: nested diff.
+
+        Yields:
+            Tuples with diff, key and subdiff for each nested diff.
 
         """
         items = diff['D'].items()
@@ -262,6 +287,7 @@ class DictHandler(TypeHandler):
 class ListHandler(TypeHandler):
     """list handler."""
 
+    extension_id = 1
     handled_type = list
 
     type_prefix = '['
@@ -273,12 +299,15 @@ class ListHandler(TypeHandler):
         self.lcs = SequenceMatcher(isjunk=None, autojunk=False)
 
     def diff(self, differ, a, b):
-        """
-        Return equality flag and diff for two lists.
+        """Calculate diff for two list objects.
 
-        :param differ: nested_diff.Differ object.
-        :param a: First list to diff.
-        :param b: Second list to diff.
+        Args:
+            differ: nested_diff.Differ object.
+            a: First list to diff.
+            b: Second list to diff.
+
+        Returns:
+            Tuple: equality flag and nested diff.
 
         >>> from nested_diff import Differ
         >>>
@@ -342,18 +371,22 @@ class ListHandler(TypeHandler):
             if equal:
                 return equal, {'U': a}
             return equal, {'D': diff}
-        elif equal and differ.op_u:
+
+        if equal and differ.op_u:
             return equal, {'U': a}
 
         return equal, {}
 
     def patch(self, patcher, target, diff):
-        """
-        Return patched list.
+        """Patch list object.
 
-        :param patcher: nested_diff.Patcher object.
-        :param target: list to patch.
-        :param diff: Nested diff.
+        Args:
+            patcher: nested_diff.Patcher object.
+            target: list to patch.
+            diff: Nested diff.
+
+        Returns:
+            Patched list.
 
         """
         i, j = 0, 0  # index, scatter
@@ -377,10 +410,14 @@ class ListHandler(TypeHandler):
         return target
 
     def iterate_diff(self, iterator, diff):
-        """
-        Iterate over list diff.
+        """Iterate over nested diff.
 
-        :param diff: nested diff.
+        Args:
+            iterator: nested_diff.Iterator object.
+            diff: Nested diff.
+
+        Yields:
+            Tuples with diff, key and subdiff for each nested diff.
 
         """
         idx = 0
@@ -420,18 +457,22 @@ class ListHandler(TypeHandler):
 class TupleHandler(ListHandler):
     """tuple handler."""
 
+    extension_id = 2
     handled_type = tuple
 
     type_prefix = '('
     type_suffix = ')'
 
     def diff(self, differ, a, b):
-        """
-        Return equality flag and diff for two tuples.
+        """Calculate diff for two tuple objects.
 
-        :param differ: nested_diff.Differ object.
-        :param a: First tuple to diff.
-        :param b: Second tuple to diff.
+        Args:
+            differ: nested_diff.Differ object.
+            a: First tuple to diff.
+            b: Second tuple to diff.
+
+        Returns:
+            Tuple: equality flag and nested diff.
 
         >>> from nested_diff import Differ
         >>>
@@ -452,12 +493,15 @@ class TupleHandler(ListHandler):
         return equal, diff
 
     def patch(self, patcher, target, diff):
-        """
-        Return patched tuple.
+        """Patch tuple object.
 
-        :param patcher: nested_diff.Patcher object.
-        :param target: tuple to patch.
-        :param diff: Nested diff.
+        Args:
+            patcher: nested_diff.Patcher object.
+            target: tuple to patch.
+            diff: Nested diff.
+
+        Returns:
+            Patched tuple.
 
         """
         return tuple(super().patch(patcher, list(target), diff))
@@ -466,15 +510,19 @@ class TupleHandler(ListHandler):
 class SetHandler(TypeHandler):
     """set handler."""
 
+    extension_id = 3
     handled_type = set
 
     def diff(self, differ, a, b):
-        """
-        Return diff for two sets.
+        """Calculate diff for two set objects.
 
-        :param differ: nested_diff.Differ object.
-        :param a: First set to diff.
-        :param b: Second set to diff.
+        Args:
+            differ: nested_diff.Differ object.
+            a: First set to diff.
+            b: Second set to diff.
+
+        Returns:
+            Tuple: equality flag and nested diff.
 
         >>> from nested_diff import Differ
         >>>
@@ -482,7 +530,7 @@ class SetHandler(TypeHandler):
         >>> b = {2, 3}
         >>>
         >>> Differ(handlers=[SetHandler()], U=False).diff(a, b)
-        (False, {'D': [{'R': 1}, {'A': 3}], 'E': set()})
+        (False, {'D': [{'R': 1}, {'A': 3}], 'E': 3})
         >>>
         """
         diff = []
@@ -503,17 +551,20 @@ class SetHandler(TypeHandler):
                 equal = False
 
         if diff:
-            return equal, {'D': diff, 'E': a.__class__()}
+            return equal, {'D': diff, 'E': self.extension_id}
 
         return equal, {}
 
     def patch(self, patcher, target, diff):
-        """
-        Return patched set.
+        """Patch set object.
 
-        :param patcher: nested_diff.Patcher object.
-        :param target: set to patch.
-        :param diff: Nested diff.
+        Args:
+            patcher: nested_diff.Patcher object.
+            target: set object to patch.
+            diff: Nested diff.
+
+        Returns:
+            Patched set.
 
         """
         for subdiff in diff['D']:
@@ -543,15 +594,19 @@ class SetHandler(TypeHandler):
 class FrozenSetHandler(SetHandler):
     """frozenset handler."""
 
+    extension_id = 4
     handled_type = frozenset
 
     def patch(self, patcher, target, diff):
-        """
-        Return patched frozenset.
+        """Patch frozenset object.
 
-        :param patcher: nested_diff.Patcher object.
-        :param target: frozenset to patch.
-        :param diff: Nested diff.
+        Args:
+            patcher: nested_diff.Patcher object.
+            target: frozenset object to patch.
+            diff: Nested diff.
+
+        Returns:
+            Patched frozenset.
 
         """
         return frozenset(super().patch(patcher, set(target), diff))
@@ -560,13 +615,14 @@ class FrozenSetHandler(SetHandler):
 class TextHandler(TypeHandler):
     """text (multiline string) handler."""
 
+    extension_id = 5
     handled_type = str
 
     def __init__(self, context=3):
-        """
-        Initialize handler.
+        """Initialize handler.
 
-        :param context: Amount of context lines.
+        Args:
+            context: Amount of context lines.
 
         """
         super().__init__()
@@ -574,15 +630,16 @@ class TextHandler(TypeHandler):
         self.context = context
 
     def diff(self, differ, a, b):
-        r"""
-        Return equality flag and diff for texts (multiline strings).
+        r"""Calculate diff for two texts (multiline strings).
 
-        Result is a unified-like diff formatted as nested diff structure, with
-        'I' tagged subdiffs containing hunks headers.
+        Args:
+            differ: nested_diff.Differ object.
+            a: First string to diff.
+            b: Second string to diff.
 
-        :param differ: nested_diff.Differ object.
-        :param a: First string to diff.
-        :param b: Second string to diff.
+        Returns:
+            Equality flag and a unified-like diff formatted as nested diff
+            structure, with 'I' tagged subdiffs containing hunks headers.
 
         >>> from nested_diff import Differ
         >>>
@@ -591,7 +648,7 @@ class TextHandler(TypeHandler):
         >>>
         >>> Differ(handlers=[TextHandler()]).diff(a, b)
         (False,
-         {'D': [{'I': [0, 1, 0, 2]}, {'U': 'one'}, {'A': 'two'}], 'E': ''})
+         {'D': [{'I': [0, 1, 0, 2]}, {'U': 'one'}, {'A': 'two'}], 'E': 5})
         >>>
         """
         lines_a = a.split('\n', -1)
@@ -628,21 +685,27 @@ class TextHandler(TypeHandler):
                     diff.extend({'A': line} for line in lines_b[j1:j2])
 
         if diff:
-            return equal, {'D': diff, 'E': a.__class__()}
+            return equal, {'D': diff, 'E': self.extension_id}
 
         return equal, {}
 
     def patch(self, patcher, target, diff):
-        """
-        Return patched text (multiline string).
+        """Patch text (multiline string).
 
         Unlike GNU patch, this algorithm does not implement any heuristics and
         patch target in straightforward way: get position from hunk header and
         apply changes specified in a hunk.
 
-        :param patcher: nested_diff.Patcher object.
-        :param target: string to patch.
-        :param diff: Nested diff.
+        Args:
+            patcher: nested_diff.Patcher object.
+            target: string to patch.
+            diff: Nested diff.
+
+        Returns:
+            Patched string.
+
+        Raises:
+            ValueError: Items and/or ops doesn't match diff/object.
 
         """
         offset = 0
